from __future__ import annotations

from collections.abc import Mapping, Sequence
from datetime import date, datetime, time
from decimal import Decimal
import inspect


def _is_scalar(value: object) -> bool:
    return value is None or isinstance(value, (bool, int, float, str))


def serialize(value: object) -> object:
    if _is_scalar(value):
        return value

    if isinstance(value, Decimal):
        return str(value)

    if isinstance(value, (datetime, date, time)):
        return value.isoformat()

    if isinstance(value, Mapping):
        return {str(key): serialize(item) for key, item in value.items()}

    if isinstance(value, Sequence) and not isinstance(value, (str, bytes, bytearray)):
        return [serialize(item) for item in value]

    if inspect.isclass(value):
        return value.__name__

    public_names = []
    for name in dir(value):
        if name.startswith("_"):
            continue
        try:
            attr = getattr(value, name)
        except Exception:
            continue
        if callable(attr):
            continue
        public_names.append(name)

    if public_names:
        return {name: serialize(getattr(value, name)) for name in public_names}

    return str(value)



from datetime import date, datetime
from decimal import Decimal

from longbridge_mcp.serialize import serialize


class SampleObject:
    foo = "bar"
    amount = Decimal("1.23")
    created = date(2026, 3, 12)


def test_serialize_nested_values():
    result = serialize(
        {
            "value": Decimal("3.14"),
            "items": [datetime(2026, 3, 12, 9, 30), SampleObject()],
        }
    )

    assert result == {
        "value": "3.14",
        "items": [
            "2026-03-12T09:30:00",
            {"amount": "1.23", "created": "2026-03-12", "foo": "bar"},
        ],
    }



import asyncio
import os

import pytest

from longbridge_mcp.server import build_server


class FakeQuoteContext:
    def static_info(self, symbols):
        return [{"symbols": symbols, "kind": "static"}]

    def quote(self, symbols):
        return [{"symbols": symbols, "kind": "quote"}]

    def depth(self, symbol):
        return {"symbol": symbol, "kind": "depth"}

    def trades(self, symbol, count):
        return [{"symbol": symbol, "count": count}]

    def intraday(self, symbol):
        return {"symbol": symbol, "kind": "intraday"}

    def history_candlesticks_by_date(self, symbol, period, adjust_type, start_date, end_date):
        return {
            "mode": "date",
            "symbol": symbol,
            "period": period.__name__,
            "adjust_type": adjust_type.__name__,
            "start_date": start_date.isoformat(),
            "end_date": end_date.isoformat(),
        }

    def history_candlesticks_by_offset(self, symbol, period, adjust_type, forward, count, anchor_datetime=None):
        return {
            "mode": "offset",
            "symbol": symbol,
            "period": period.__name__,
            "adjust_type": adjust_type.__name__,
            "forward": forward,
            "count": count,
            "anchor_datetime": anchor_datetime.isoformat() if anchor_datetime else None,
        }

    def capital_flow(self, symbol):
        return {"symbol": symbol}

    def capital_distribution(self, symbol):
        return {"symbol": symbol}

    def calc_indexes(self, symbols, indexes):
        return {"symbols": symbols, "indexes": [item.__name__ for item in indexes]}

    def watchlist(self):
        return [{"id": 1, "name": "watch"}]

    def security_list(self, market, category=None):
        return {"market": str(market).split(".")[-1], "category": None if category is None else str(category).split(".")[-1]}

    def market_temperature(self, market):
        return {"market": str(market).split(".")[-1]}

    def history_market_temperature(self, market, start_date, end_date):
        return {"market": str(market).split(".")[-1], "start_date": start_date.isoformat(), "end_date": end_date.isoformat()}

    def trading_days(self, market, begin, end):
        return {"market": str(market).split(".")[-1], "begin": begin.isoformat(), "end": end.isoformat()}

    def option_chain_expiry_date_list(self, symbol):
        return [{"symbol": symbol}]

    def option_chain_info_by_date(self, symbol, expiry_date):
        return [{"symbol": symbol, "expiry_date": expiry_date.isoformat()}]

    def option_quote(self, symbols):
        return [{"symbols": symbols}]

    def warrant_quote(self, symbols):
        return [{"symbols": symbols}]

    def warrant_list(self, symbol, sort_by, sort_order, warrant_types, issuer_ids, expiry_dates, price_types, statuses):
        return {
            "symbol": symbol,
            "sort_by": str(sort_by).split(".")[-1],
            "sort_order": str(sort_order).split(".")[-1],
            "statuses": None if statuses is None else [str(item).split(".")[-1] for item in statuses],
        }

    def brokers(self, symbol):
        return {"symbol": symbol}

    def participants(self):
        return [{"id": 1}]

    def quote_level(self):
        return "LV1"

    def quote_package_details(self):
        return [{"package": "basic"}]

    def create_watchlist_group(self, name, securities=None):
        return {"name": name, "securities": securities}

    def update_watchlist_group(self, id, name=None, securities=None, mode=None):
        return {"id": id, "name": name, "securities": securities, "mode": None if mode is None else mode.__name__}

    def delete_watchlist_group(self, id, purge=False):
        return {"id": id, "purge": purge}


class FakeTradeContext:
    def history_executions(self, symbol=None, start_at=None, end_at=None):
        return [{"symbol": symbol, "start_at": None if start_at is None else start_at.isoformat()}]

    def today_executions(self, symbol=None, order_id=None):
        return [{"symbol": symbol, "order_id": order_id}]

    def account_balance(self, currency=None):
        return [{"currency": currency or "USD"}]

    def stock_positions(self, symbols=None):
        return {"symbols": symbols}

    def cash_flow(self, start_at, end_at, business_type=None, symbol=None, page=None, size=None):
        return {
            "start_at": start_at.isoformat(),
            "end_at": end_at.isoformat(),
            "business_type": None if business_type is None else business_type.__name__,
            "symbol": symbol,
            "page": page,
            "size": size,
        }

    def fund_positions(self, symbols=None):
        return {"symbols": symbols}

    def history_orders(self, symbol=None, status=None, side=None, market=None, start_at=None, end_at=None):
        return {
            "symbol": symbol,
            "status": None if status is None else [str(item).split(".")[-1] for item in status],
            "side": None if side is None else str(side).split(".")[-1],
            "market": None if market is None else str(market).split(".")[-1],
        }

    def today_orders(self, symbol=None, status=None, side=None, market=None, order_id=None):
        return {
            "symbol": symbol,
            "status": None if status is None else [str(item).split(".")[-1] for item in status],
            "side": None if side is None else str(side).split(".")[-1],
            "market": None if market is None else str(market).split(".")[-1],
            "order_id": order_id,
        }

    def order_detail(self, order_id):
        return {"order_id": order_id}

    def margin_ratio(self, symbol):
        return {"symbol": symbol}

    def submit_order(self, *args):
        return {"args": list(args)}

    def cancel_order(self, order_id):
        return {"order_id": order_id}

    def replace_order(self, *args):
        return {"args": list(args)}


class FakeService:
    def __init__(self):
        self.quote_context = FakeQuoteContext()
        self.trade_context = FakeTradeContext()

    def call(self, fn, *args, **kwargs):
        return fn(*args, **kwargs)


def tool_names(server):
    return {tool.name for tool in asyncio.run(server.list_tools())}


def call_result(server, name, arguments):
    content, structured = asyncio.run(server.call_tool(name, arguments))
    assert content
    return structured["result"]


def test_default_server_omits_write_tools(monkeypatch):
    monkeypatch.delenv("LONGBRIDGE_MCP_ENABLE_WRITE_TOOLS", raising=False)
    server = build_server(FakeService())
    names = tool_names(server)
    assert "quote-static-info" in names
    assert "trade-submit-order" not in names


def test_write_enabled_server_registers_write_tools(monkeypatch):
    monkeypatch.setenv("LONGBRIDGE_MCP_ENABLE_WRITE_TOOLS", "true")
    server = build_server(FakeService())
    names = tool_names(server)
    assert "trade-submit-order" in names
    assert "quote-watch-list-delete-group" in names


def test_call_legacy_quote_tool(monkeypatch):
    monkeypatch.delenv("LONGBRIDGE_MCP_ENABLE_WRITE_TOOLS", raising=False)
    server = build_server(FakeService())
    result = call_result(server, "quote-static-info", {"symbols": ["AAPL.US"]})
    assert result == [{"symbols": ["AAPL.US"], "kind": "static"}]


def test_call_extended_quote_tool(monkeypatch):
    monkeypatch.delenv("LONGBRIDGE_MCP_ENABLE_WRITE_TOOLS", raising=False)
    server = build_server(FakeService())
    result = call_result(server, "quote-security-list", {"market": "us"})
    assert result == {"market": "US", "category": None}


def test_call_trade_read_tool(monkeypatch):
    monkeypatch.delenv("LONGBRIDGE_MCP_ENABLE_WRITE_TOOLS", raising=False)
    server = build_server(FakeService())
    result = call_result(
        server,
        "trade-history-orders",
        {"statuses": ["filled"], "side": "buy", "market": "us"},
    )
    assert result == {"symbol": None, "status": ["Filled"], "side": "Buy", "market": "US"}


@pytest.mark.skipif(
    not all(os.getenv(name) for name in ("LONGBRIDGE_APP_KEY", "LONGBRIDGE_APP_SECRET", "LONGBRIDGE_ACCESS_TOKEN")),
    reason="Longbridge credentials not configured",
)
def test_live_smoke_import():
    server = build_server()
    assert "quote-static-info" in tool_names(server)



from longbridge_mcp.config import load_settings


def test_load_settings_defaults_to_read_only():
    settings = load_settings({})
    assert settings.enable_write_tools is False


def test_load_settings_parses_true():
    settings = load_settings({"LONGBRIDGE_MCP_ENABLE_WRITE_TOOLS": "true"})
    assert settings.enable_write_tools is True



import pytest

from longbridge_mcp.validation import HistoryCandlesticksInput, OrdersQueryInput, SymbolListInput


def test_history_candlesticks_requires_date_range_for_by_date():
    with pytest.raises(ValueError):
        HistoryCandlesticksInput(
            symbol="AAPL.US",
            period="1m",
            adjust_type="no_adjust",
            query_type="by_date",
        )


def test_history_candlesticks_accepts_aliases():
    payload = HistoryCandlesticksInput(
        symbol="AAPL.US",
        period="1m",
        adjust_type="forward_adjust",
        query_type="by_offset",
        forward=True,
    )

    assert str(payload.sdk_period()) == "Period.Min_1"
    assert str(payload.sdk_adjust_type()) == "AdjustType.ForwardAdjust"


def test_symbol_list_limit_is_enforced():
    with pytest.raises(ValueError):
        SymbolListInput(symbols=["AAPL.US"] * 501)


def test_orders_query_maps_enum_filters():
    payload = OrdersQueryInput(statuses=["filled"], side="buy", market="us")
    assert str(payload.sdk_statuses()[0]) == "OrderStatus.Filled"
    assert str(payload.sdk_side()) == "OrderSide.Buy"
    assert str(payload.sdk_market()) == "Market.US"



from __future__ import annotations

import os
from dataclasses import dataclass

from longbridge.openapi import Config


def _parse_bool(value: str | None, *, default: bool) -> bool:
    if value is None:
        return default
    normalized = value.strip().lower()
    if normalized in {"1", "true", "yes", "on"}:
        return True
    if normalized in {"0", "false", "no", "off"}:
        return False
    raise ValueError(
        "LONGBRIDGE_MCP_ENABLE_WRITE_TOOLS must be one of: true, false, 1, 0, yes, no, on, off"
    )


@dataclass(frozen=True)
class MCPSettings:
    enable_write_tools: bool = False


def load_settings(environ: dict[str, str] | None = None) -> MCPSettings:
    values = environ or os.environ
    return MCPSettings(
        enable_write_tools=_parse_bool(
            values.get("LONGBRIDGE_MCP_ENABLE_WRITE_TOOLS"),
            default=False,
        )
    )


def load_longbridge_config() -> Config:
    try:
        return Config.from_apikey_env()
    except Exception as exc:  # pragma: no cover - exact SDK exception type is binding-defined
        raise RuntimeError(
            "Failed to load Longbridge configuration from environment. "
            "Set LONGBRIDGE_APP_KEY, LONGBRIDGE_APP_SECRET, and LONGBRIDGE_ACCESS_TOKEN."
        ) from exc



from __future__ import annotations

from collections.abc import Callable
from typing import Any

from longbridge.openapi import QuoteContext, TradeContext
from mcp.server.fastmcp import FastMCP
from pydantic import ValidationError

from .config import load_longbridge_config, load_settings
from .serialize import serialize
from .validation import (
    AccountBalanceInput,
    CalcIndexesInput,
    CancelOrderInput,
    CashFlowInput,
    CreateWatchlistGroupInput,
    DeleteWatchlistGroupInput,
    HistoryCandlesticksInput,
    MarginRatioInput,
    MarketDateRangeInput,
    MarketInput,
    OptionalSymbolsInput,
    OptionChainInfoInput,
    OrderDetailInput,
    OrdersQueryInput,
    QuoteTradesInput,
    ReplaceOrderInput,
    SecurityListInput,
    SingleSymbolInput,
    SubmitOrderInput,
    SymbolListInput,
    TodayExecutionsInput,
    TodayOrdersInput,
    TradeHistoryExecutionsInput,
    TradingDaysInput,
    UpdateWatchlistGroupInput,
    WarrantListInput,
)


class LongbridgeService:
    def __init__(self) -> None:
        self._config = None
        self._quote_context: QuoteContext | None = None
        self._trade_context: TradeContext | None = None

    @property
    def config(self):
        if self._config is None:
            self._config = load_longbridge_config()
        return self._config

    @property
    def quote_context(self) -> QuoteContext:
        if self._quote_context is None:
            self._quote_context = QuoteContext(self.config)
        return self._quote_context

    @property
    def trade_context(self) -> TradeContext:
        if self._trade_context is None:
            self._trade_context = TradeContext(self.config)
        return self._trade_context

    def call(self, fn: Callable[..., Any], *args: Any, **kwargs: Any) -> Any:
        return serialize(fn(*args, **kwargs))


StructuredResult = dict[str, object] | list[object] | str | int | float | bool | None


def _validated(model_cls: type[Any], **values: Any) -> Any:
    try:
        return model_cls(**values)
    except ValidationError as exc:
        raise ValueError(exc.errors(include_url=False)) from exc


def _register_tool(
    mcp: FastMCP,
    fn: Callable[..., Any],
    *,
    name: str,
    description: str,
) -> None:
    mcp.add_tool(
        fn,
        name=name,
        description=description,
        structured_output=True,
    )


def build_server(service: LongbridgeService | None = None) -> FastMCP:
    settings = load_settings()
    gateway = service or LongbridgeService()
    mcp = FastMCP(
        name="Longbridge",
        instructions=(
            "Longbridge OpenAPI MCP server. Use the read-only quote and account tools by default. "
            "Write tools are available only when LONGBRIDGE_MCP_ENABLE_WRITE_TOOLS=true."
        ),
    )

    def quote_static_info(symbols: list[str]) -> StructuredResult:
        params = _validated(SymbolListInput, symbols=symbols)
        return gateway.call(gateway.quote_context.static_info, params.symbols)

    def quote_realtime_info(symbols: list[str]) -> StructuredResult:
        params = _validated(SymbolListInput, symbols=symbols)
        return gateway.call(gateway.quote_context.quote, params.symbols)

    def quote_depth(symbol: str) -> StructuredResult:
        params = _validated(SingleSymbolInput, symbol=symbol)
        return gateway.call(gateway.quote_context.depth, params.symbol)

    def quote_trades(symbol: str, count: int) -> StructuredResult:
        params = _validated(QuoteTradesInput, symbol=symbol, count=count)
        return gateway.call(gateway.quote_context.trades, params.symbol, params.count)

    def quote_intraday(symbol: str) -> StructuredResult:
        params = _validated(SingleSymbolInput, symbol=symbol)
        return gateway.call(gateway.quote_context.intraday, params.symbol)

    def quote_history_candlesticks(
        symbol: str,
        period: str,
        query_type: str,
        adjust_type: str = "NoAdjust",
        start_date: str | None = None,
        end_date: str | None = None,
        forward: bool | None = None,
        count: int = 10,
        anchor_datetime: str | None = None,
    ) -> StructuredResult:
        params = _validated(
            HistoryCandlesticksInput,
            symbol=symbol,
            period=period,
            query_type=query_type,
            adjust_type=adjust_type,
            start_date=start_date,
            end_date=end_date,
            forward=forward,
            count=count,
            anchor_datetime=anchor_datetime,
        )
        if params.query_type == "by_date":
            return gateway.call(
                gateway.quote_context.history_candlesticks_by_date,
                params.symbol,
                params.sdk_period(),
                params.sdk_adjust_type(),
                params.start_date,
                params.end_date,
            )
        return gateway.call(
            gateway.quote_context.history_candlesticks_by_offset,
            params.symbol,
            params.sdk_period(),
            params.sdk_adjust_type(),
            params.forward,
            params.count,
            params.anchor_datetime,
        )

    def quote_capital_flow(symbol: str) -> StructuredResult:
        params = _validated(SingleSymbolInput, symbol=symbol)
        return gateway.call(gateway.quote_context.capital_flow, params.symbol)

    def quote_capital_distribution(symbol: str) -> StructuredResult:
        params = _validated(SingleSymbolInput, symbol=symbol)
        return gateway.call(gateway.quote_context.capital_distribution, params.symbol)

    def quote_calc_index(symbols: list[str], indexes: list[str]) -> StructuredResult:
        params = _validated(CalcIndexesInput, symbols=symbols, indexes=indexes)
        return gateway.call(gateway.quote_context.calc_indexes, params.symbols, params.sdk_indexes())

    def quote_watch_list() -> StructuredResult:
        return gateway.call(gateway.quote_context.watchlist)

    def quote_security_list(market: str, category: str | None = None) -> StructuredResult:
        params = _validated(SecurityListInput, market=market, category=category)
        return gateway.call(gateway.quote_context.security_list, params.sdk_market(), params.sdk_category())

    def quote_market_temperature(market: str) -> StructuredResult:
        params = _validated(MarketInput, market=market)
        return gateway.call(gateway.quote_context.market_temperature, params.sdk_market())

    def quote_history_market_temperature(market: str, start_date: str, end_date: str) -> StructuredResult:
        params = _validated(MarketDateRangeInput, market=market, start_date=start_date, end_date=end_date)
        return gateway.call(
            gateway.quote_context.history_market_temperature,
            params.sdk_market(),
            params.start_date,
            params.end_date,
        )

    def quote_trading_days(market: str, begin: str, end: str) -> StructuredResult:
        params = _validated(TradingDaysInput, market=market, begin=begin, end=end)
        return gateway.call(gateway.quote_context.trading_days, params.sdk_market(), params.begin, params.end)

    def quote_option_chain_expiry_dates(symbol: str) -> StructuredResult:
        params = _validated(SingleSymbolInput, symbol=symbol)
        return gateway.call(gateway.quote_context.option_chain_expiry_date_list, params.symbol)

    def quote_option_chain_info(symbol: str, expiry_date: str) -> StructuredResult:
        params = _validated(OptionChainInfoInput, symbol=symbol, expiry_date=expiry_date)
        return gateway.call(gateway.quote_context.option_chain_info_by_date, params.symbol, params.expiry_date)

    def quote_option_quote(symbols: list[str]) -> StructuredResult:
        params = _validated(SymbolListInput, symbols=symbols)
        return gateway.call(gateway.quote_context.option_quote, params.symbols)

    def quote_warrant_quote(symbols: list[str]) -> StructuredResult:
        params = _validated(SymbolListInput, symbols=symbols)
        return gateway.call(gateway.quote_context.warrant_quote, params.symbols)

    def quote_warrant_list(
        symbol: str,
        sort_by: str,
        sort_order: str,
        warrant_types: list[str] | None = None,
        issuer_ids: list[int] | None = None,
        expiry_dates: list[str] | None = None,
        price_types: list[str] | None = None,
        statuses: list[str] | None = None,
    ) -> StructuredResult:
        params = _validated(
            WarrantListInput,
            symbol=symbol,
            sort_by=sort_by,
            sort_order=sort_order,
            warrant_types=warrant_types,
            issuer_ids=issuer_ids,
            expiry_dates=expiry_dates,
            price_types=price_types,
            statuses=statuses,
        )
        return gateway.call(
            gateway.quote_context.warrant_list,
            params.symbol,
            params.sdk_sort_by(),
            params.sdk_sort_order(),
            params.sdk_warrant_types(),
            params.issuer_ids,
            params.sdk_expiry_dates(),
            params.sdk_price_types(),
            params.sdk_statuses(),
        )

    def quote_brokers(symbol: str) -> StructuredResult:
        params = _validated(SingleSymbolInput, symbol=symbol)
        return gateway.call(gateway.quote_context.brokers, params.symbol)

    def quote_participants() -> StructuredResult:
        return gateway.call(gateway.quote_context.participants)

    def quote_level() -> StructuredResult:
        return gateway.call(gateway.quote_context.quote_level)

    def quote_package_details() -> StructuredResult:
        return gateway.call(gateway.quote_context.quote_package_details)

    def trade_history_executions(
        symbol: str | None = None,
        start_at: str | None = None,
        end_at: str | None = None,
    ) -> StructuredResult:
        params = _validated(TradeHistoryExecutionsInput, symbol=symbol, start_at=start_at, end_at=end_at)
        return gateway.call(
            gateway.trade_context.history_executions,
            params.symbol,
            params.start_at,
            params.end_at,
        )

    def trade_today_executions(symbol: str | None = None, order_id: str | None = None) -> StructuredResult:
        params = _validated(TodayExecutionsInput, symbol=symbol, order_id=order_id)
        return gateway.call(gateway.trade_context.today_executions, params.symbol, params.order_id)

    def trade_account_balance(currency: str | None = None) -> StructuredResult:
        params = _validated(AccountBalanceInput, currency=currency)
        return gateway.call(gateway.trade_context.account_balance, params.currency)

    def trade_stock_positions(symbols: list[str] | None = None) -> StructuredResult:
        params = _validated(OptionalSymbolsInput, symbols=symbols)
        return gateway.call(gateway.trade_context.stock_positions, params.symbols)

    def trade_cash_flow(
        start_at: str,
        end_at: str,
        business_type: str | None = None,
        symbol: str | None = None,
        page: int | None = None,
        size: int | None = None,
    ) -> StructuredResult:
        params = _validated(
            CashFlowInput,
            start_at=start_at,
            end_at=end_at,
            business_type=business_type,
            symbol=symbol,
            page=page,
            size=size,
        )
        return gateway.call(
            gateway.trade_context.cash_flow,
            params.start_at,
            params.end_at,
            params.sdk_business_type(),
            params.symbol,
            params.page,
            params.size,
        )

    def trade_fund_positions(symbols: list[str] | None = None) -> StructuredResult:
        params = _validated(OptionalSymbolsInput, symbols=symbols)
        return gateway.call(gateway.trade_context.fund_positions, params.symbols)

    def trade_history_orders(
        symbol: str | None = None,
        statuses: list[str] | None = None,
        side: str | None = None,
        market: str | None = None,
        start_at: str | None = None,
        end_at: str | None = None,
    ) -> StructuredResult:
        params = _validated(
            OrdersQueryInput,
            symbol=symbol,
            statuses=statuses,
            side=side,
            market=market,
            start_at=start_at,
            end_at=end_at,
        )
        return gateway.call(
            gateway.trade_context.history_orders,
            params.symbol,
            params.sdk_statuses(),
            params.sdk_side(),
            params.sdk_market(),
            params.start_at,
            params.end_at,
        )

    def trade_today_orders(
        symbol: str | None = None,
        statuses: list[str] | None = None,
        side: str | None = None,
        market: str | None = None,
        order_id: str | None = None,
    ) -> StructuredResult:
        params = _validated(
            TodayOrdersInput,
            symbol=symbol,
            statuses=statuses,
            side=side,
            market=market,
            order_id=order_id,
        )
        return gateway.call(
            gateway.trade_context.today_orders,
            params.symbol,
            params.sdk_statuses(),
            params.sdk_side(),
            params.sdk_market(),
            params.order_id,
        )

    def trade_order_detail(order_id: str) -> StructuredResult:
        params = _validated(OrderDetailInput, order_id=order_id)
        return gateway.call(gateway.trade_context.order_detail, params.order_id)

    def trade_margin_ratio(symbol: str) -> StructuredResult:
        params = _validated(MarginRatioInput, symbol=symbol)
        return gateway.call(gateway.trade_context.margin_ratio, params.symbol)

    read_tools = [
        ("quote-static-info", "Get basic information for one or more securities.", quote_static_info),
        ("quote-realtime-info", "Get real-time quotes for one or more securities.", quote_realtime_info),
        ("quote-depth", "Get order book depth for a security.", quote_depth),
        ("quote-trades", "Get recent trades for a security.", quote_trades),
        ("quote-intraday", "Get intraday line data for a security.", quote_intraday),
        ("quote-history-candlesticks", "Get historical candlesticks by date range or offset.", quote_history_candlesticks),
        ("quote-capital-flow", "Get intraday capital flow for a security.", quote_capital_flow),
        ("quote-capital-distribution", "Get capital distribution for a security.", quote_capital_distribution),
        ("quote-calc-index", "Calculate quote indexes for one or more securities.", quote_calc_index),
        ("quote-watch-list", "Get watch list groups.", quote_watch_list),
        ("quote-security-list", "Get the market security list.", quote_security_list),
        ("quote-market-temperature", "Get current market temperature.", quote_market_temperature),
        ("quote-history-market-temperature", "Get historical market temperature.", quote_history_market_temperature),
        ("quote-trading-days", "Get trading days for a market date range.", quote_trading_days),
        ("quote-option-chain-expiry-dates", "Get option expiry dates for an underlying symbol.", quote_option_chain_expiry_dates),
        ("quote-option-chain-info", "Get option chain details for a symbol and expiry date.", quote_option_chain_info),
        ("quote-option-quote", "Get option quotes.", quote_option_quote),
        ("quote-warrant-quote", "Get warrant quotes.", quote_warrant_quote),
        ("quote-warrant-list", "Get warrant list results for an underlying symbol.", quote_warrant_list),
        ("quote-brokers", "Get brokers for a security.", quote_brokers),
        ("quote-participants", "Get market participants.", quote_participants),
        ("quote-level", "Get your quote level.", quote_level),
        ("quote-package-details", "Get quote package details.", quote_package_details),
        ("trade-history-executions", "Get historical executions.", trade_history_executions),
        ("trade-today-executions", "Get today's executions.", trade_today_executions),
        ("trade-account-balance", "Get account balance.", trade_account_balance),
        ("trade-stock-positions", "Get stock positions.", trade_stock_positions),
        ("trade-cash-flow", "Get cash flow records.", trade_cash_flow),
        ("trade-fund-positions", "Get fund positions.", trade_fund_positions),
        ("trade-history-orders", "Get historical orders.", trade_history_orders),
        ("trade-today-orders", "Get today's orders.", trade_today_orders),
        ("trade-order-detail", "Get order details.", trade_order_detail),
        ("trade-margin-ratio", "Get margin ratio for a symbol.", trade_margin_ratio),
    ]

    for name, description, fn in read_tools:
        _register_tool(mcp, fn, name=name, description=description)

    if settings.enable_write_tools:
        def trade_submit_order(
            symbol: str,
            order_type: str,
            side: str,
            submitted_quantity: str,
            time_in_force: str,
            submitted_price: str | None = None,
            trigger_price: str | None = None,
            limit_offset: str | None = None,
            trailing_amount: str | None = None,
            trailing_percent: str | None = None,
            expire_date: str | None = None,
            outside_rth: str | None = None,
            limit_depth_level: int | None = None,
            trigger_count: int | None = None,
            monitor_price: str | None = None,
            remark: str | None = None,
        ) -> StructuredResult:
            params = _validated(
                SubmitOrderInput,
                symbol=symbol,
                order_type=order_type,
                side=side,
                submitted_quantity=submitted_quantity,
                time_in_force=time_in_force,
                submitted_price=submitted_price,
                trigger_price=trigger_price,
                limit_offset=limit_offset,
                trailing_amount=trailing_amount,
                trailing_percent=trailing_percent,
                expire_date=expire_date,
                outside_rth=outside_rth,
                limit_depth_level=limit_depth_level,
                trigger_count=trigger_count,
                monitor_price=monitor_price,
                remark=remark,
            )
            return gateway.call(
                gateway.trade_context.submit_order,
                params.symbol,
                params.sdk_order_type(),
                params.sdk_side(),
                params.submitted_quantity,
                params.sdk_time_in_force(),
                params.submitted_price,
                params.trigger_price,
                params.limit_offset,
                params.trailing_amount,
                params.trailing_percent,
                params.expire_date,
                params.sdk_outside_rth(),
                params.limit_depth_level,
                params.trigger_count,
                params.monitor_price,
                params.remark,
            )

        def trade_cancel_order(order_id: str) -> StructuredResult:
            params = _validated(CancelOrderInput, order_id=order_id)
            return gateway.call(gateway.trade_context.cancel_order, params.order_id)

        def trade_replace_order(
            order_id: str,
            quantity: str,
            price: str | None = None,
            trigger_price: str | None = None,
            limit_offset: str | None = None,
            trailing_amount: str | None = None,
            trailing_percent: str | None = None,
            limit_depth_level: int | None = None,
            trigger_count: int | None = None,
            monitor_price: str | None = None,
            remark: str | None = None,
        ) -> StructuredResult:
            params = _validated(
                ReplaceOrderInput,
                order_id=order_id,
                quantity=quantity,
                price=price,
                trigger_price=trigger_price,
                limit_offset=limit_offset,
                trailing_amount=trailing_amount,
                trailing_percent=trailing_percent,
                limit_depth_level=limit_depth_level,
                trigger_count=trigger_count,
                monitor_price=monitor_price,
                remark=remark,
            )
            return gateway.call(
                gateway.trade_context.replace_order,
                params.order_id,
                params.quantity,
                params.price,
                params.trigger_price,
                params.limit_offset,
                params.trailing_amount,
                params.trailing_percent,
                params.limit_depth_level,
                params.trigger_count,
                params.monitor_price,
                params.remark,
            )

        def quote_watch_list_create_group(name: str, securities: list[str] | None = None) -> StructuredResult:
            params = _validated(CreateWatchlistGroupInput, name=name, securities=securities)
            return gateway.call(gateway.quote_context.create_watchlist_group, params.name, params.securities)

        def quote_watch_list_update_group(
            id: int,
            name: str | None = None,
            securities: list[str] | None = None,
            mode: str | None = None,
        ) -> StructuredResult:
            params = _validated(
                UpdateWatchlistGroupInput,
                id=id,
                name=name,
                securities=securities,
                mode=mode,
            )
            return gateway.call(
                gateway.quote_context.update_watchlist_group,
                params.id,
                params.name,
                params.securities,
                params.sdk_mode(),
            )

        def quote_watch_list_delete_group(id: int, purge: bool = False) -> StructuredResult:
            params = _validated(DeleteWatchlistGroupInput, id=id, purge=purge)
            return gateway.call(gateway.quote_context.delete_watchlist_group, params.id, params.purge)

        write_tools = [
            ("trade-submit-order", "Submit a trade order.", trade_submit_order),
            ("trade-cancel-order", "Cancel a trade order.", trade_cancel_order),
            ("trade-replace-order", "Replace a trade order.", trade_replace_order),
            ("quote-watch-list-create-group", "Create a watch list group.", quote_watch_list_create_group),
            ("quote-watch-list-update-group", "Update a watch list group.", quote_watch_list_update_group),
            ("quote-watch-list-delete-group", "Delete a watch list group.", quote_watch_list_delete_group),
        ]
        for name, description, fn in write_tools:
            _register_tool(mcp, fn, name=name, description=description)

    return mcp


def main() -> None:
    build_server().run(transport="stdio")



from __future__ import annotations

from datetime import date, datetime
from decimal import Decimal
import re
from typing import Any, Literal

from longbridge.openapi import (
    AdjustType,
    BalanceType,
    CalcIndex,
    FilterWarrantExpiryDate,
    FilterWarrantInOutBoundsType,
    Market,
    OrderSide,
    OrderStatus,
    OrderType,
    OutsideRTH,
    Period,
    SecuritiesUpdateMode,
    SecurityListCategory,
    SortOrderType,
    TimeInForceType,
    WarrantSortBy,
    WarrantStatus,
    WarrantType,
)
from pydantic import BaseModel, ConfigDict, Field, field_validator, model_validator

SYMBOL_RE = re.compile(r"^[A-Z0-9]+(?:\.[A-Z]+)+$")


def normalize_name(value: str) -> str:
    return re.sub(r"[\s_\-]+", "", value).lower()


def enum_members(enum_cls: type[Any]) -> dict[str, Any]:
    members: dict[str, Any] = {}
    for name in dir(enum_cls):
        if name.startswith("_"):
            continue
        attr = getattr(enum_cls, name)
        if isinstance(attr, enum_cls):
            members[name] = attr
    return members


def map_enum(enum_cls: type[Any], value: str, *, aliases: dict[str, str] | None = None) -> Any:
    members = enum_members(enum_cls)
    if value in members:
        return members[value]
    normalized = normalize_name(value)
    for name, member in members.items():
        if normalize_name(name) == normalized:
            return member
    if aliases:
        alias_target = aliases.get(normalized)
        if alias_target and alias_target in members:
            return members[alias_target]
    valid = ", ".join(sorted(members))
    raise ValueError(f"Invalid value {value!r}. Expected one of: {valid}")


def map_enums(enum_cls: type[Any], values: list[str] | None, *, aliases: dict[str, str] | None = None) -> list[Any] | None:
    if values is None:
        return None
    return [map_enum(enum_cls, value, aliases=aliases) for value in values]


def validate_symbol(symbol: str) -> str:
    if not SYMBOL_RE.fullmatch(symbol):
        raise ValueError('Symbol must look like "AAPL.US"')
    return symbol


class Model(BaseModel):
    model_config = ConfigDict(extra="forbid", str_strip_whitespace=True)


class SymbolListInput(Model):
    symbols: list[str] = Field(min_length=1, max_length=500)

    @field_validator("symbols")
    @classmethod
    def _validate_symbols(cls, value: list[str]) -> list[str]:
        return [validate_symbol(item) for item in value]


class SingleSymbolInput(Model):
    symbol: str

    @field_validator("symbol")
    @classmethod
    def _validate_symbol(cls, value: str) -> str:
        return validate_symbol(value)


class QuoteTradesInput(SingleSymbolInput):
    count: int = Field(ge=1, le=1000)


class HistoryCandlesticksInput(SingleSymbolInput):
    period: str
    adjust_type: str = "NoAdjust"
    query_type: Literal["by_offset", "by_date"]
    start_date: date | None = None
    end_date: date | None = None
    forward: bool | None = None
    count: int = Field(default=10, ge=1, le=1000)
    anchor_datetime: datetime | None = None

    @model_validator(mode="after")
    def _validate_query_mode(self) -> "HistoryCandlesticksInput":
        if self.query_type == "by_date":
            if not self.start_date or not self.end_date:
                raise ValueError("start_date and end_date are required when query_type='by_date'")
        else:
            if self.forward is None:
                raise ValueError("forward is required when query_type='by_offset'")
        return self

    def sdk_period(self) -> Any:
        return map_enum(
            Period,
            self.period,
            aliases={
                "1m": "Min_1",
                "2m": "Min_2",
                "3m": "Min_3",
                "5m": "Min_5",
                "10m": "Min_10",
                "15m": "Min_15",
                "20m": "Min_20",
                "30m": "Min_30",
                "45m": "Min_45",
                "60m": "Min_60",
                "120m": "Min_120",
                "180m": "Min_180",
                "240m": "Min_240",
            },
        )

    def sdk_adjust_type(self) -> Any:
        return map_enum(
            AdjustType,
            self.adjust_type,
            aliases={"noadjust": "NoAdjust", "forwardadjust": "ForwardAdjust"},
        )


class CalcIndexesInput(SymbolListInput):
    indexes: list[str] = Field(min_length=1)

    def sdk_indexes(self) -> list[Any]:
        return map_enums(CalcIndex, self.indexes)


class SecurityListInput(Model):
    market: str
    category: str | None = None

    def sdk_market(self) -> Any:
        return map_enum(Market, self.market)

    def sdk_category(self) -> Any:
        if self.category is None:
            return None
        return map_enum(SecurityListCategory, self.category)


class MarketInput(Model):
    market: str

    def sdk_market(self) -> Any:
        return map_enum(Market, self.market)


class MarketDateRangeInput(MarketInput):
    start_date: date
    end_date: date


class TradingDaysInput(MarketInput):
    begin: date
    end: date


class OptionChainInfoInput(SingleSymbolInput):
    expiry_date: date


class WarrantListInput(SingleSymbolInput):
    sort_by: str
    sort_order: str
    warrant_types: list[str] | None = None
    issuer_ids: list[int] | None = None
    expiry_dates: list[str] | None = None
    price_types: list[str] | None = None
    statuses: list[str] | None = None

    def sdk_sort_by(self) -> Any:
        return map_enum(WarrantSortBy, self.sort_by)

    def sdk_sort_order(self) -> Any:
        return map_enum(SortOrderType, self.sort_order)

    def sdk_warrant_types(self) -> list[Any] | None:
        return map_enums(WarrantType, self.warrant_types)

    def sdk_expiry_dates(self) -> list[Any] | None:
        return map_enums(FilterWarrantExpiryDate, self.expiry_dates)

    def sdk_price_types(self) -> list[Any] | None:
        return map_enums(FilterWarrantInOutBoundsType, self.price_types)

    def sdk_statuses(self) -> list[Any] | None:
        return map_enums(WarrantStatus, self.statuses)


class TradeHistoryExecutionsInput(Model):
    symbol: str | None = None
    start_at: datetime | None = None
    end_at: datetime | None = None

    @field_validator("symbol")
    @classmethod
    def _validate_symbol(cls, value: str | None) -> str | None:
        if value is None:
            return value
        return validate_symbol(value)


class TodayExecutionsInput(Model):
    symbol: str | None = None
    order_id: str | None = None

    @field_validator("symbol")
    @classmethod
    def _validate_symbol(cls, value: str | None) -> str | None:
        if value is None:
            return value
        return validate_symbol(value)


class AccountBalanceInput(Model):
    currency: str | None = None


class OptionalSymbolsInput(Model):
    symbols: list[str] | None = Field(default=None, max_length=500)

    @field_validator("symbols")
    @classmethod
    def _validate_symbols(cls, value: list[str] | None) -> list[str] | None:
        if value is None:
            return value
        return [validate_symbol(item) for item in value]


class CashFlowInput(Model):
    start_at: datetime
    end_at: datetime
    business_type: str | None = None
    symbol: str | None = None
    page: int | None = Field(default=None, ge=1)
    size: int | None = Field(default=None, ge=1, le=200)

    @field_validator("symbol")
    @classmethod
    def _validate_symbol(cls, value: str | None) -> str | None:
        if value is None:
            return value
        return validate_symbol(value)

    def sdk_business_type(self) -> Any:
        if self.business_type is None:
            return None
        return map_enum(BalanceType, self.business_type)


class OrdersQueryInput(Model):
    symbol: str | None = None
    statuses: list[str] | None = None
    side: str | None = None
    market: str | None = None
    start_at: datetime | None = None
    end_at: datetime | None = None

    @field_validator("symbol")
    @classmethod
    def _validate_symbol(cls, value: str | None) -> str | None:
        if value is None:
            return value
        return validate_symbol(value)

    def sdk_statuses(self) -> list[Any] | None:
        return map_enums(OrderStatus, self.statuses)

    def sdk_side(self) -> Any:
        if self.side is None:
            return None
        return map_enum(OrderSide, self.side)

    def sdk_market(self) -> Any:
        if self.market is None:
            return None
        return map_enum(Market, self.market)


class TodayOrdersInput(OrdersQueryInput):
    order_id: str | None = None


class OrderDetailInput(Model):
    order_id: str


class MarginRatioInput(SingleSymbolInput):
    pass


class SubmitOrderInput(Model):
    symbol: str
    order_type: str
    side: str
    submitted_quantity: Decimal
    time_in_force: str
    submitted_price: Decimal | None = None
    trigger_price: Decimal | None = None
    limit_offset: Decimal | None = None
    trailing_amount: Decimal | None = None
    trailing_percent: Decimal | None = None
    expire_date: date | None = None
    outside_rth: str | None = None
    limit_depth_level: int | None = None
    trigger_count: int | None = None
    monitor_price: Decimal | None = None
    remark: str | None = None

    @field_validator("symbol")
    @classmethod
    def _validate_symbol(cls, value: str) -> str:
        return validate_symbol(value)

    def sdk_order_type(self) -> Any:
        return map_enum(OrderType, self.order_type)

    def sdk_side(self) -> Any:
        return map_enum(OrderSide, self.side)

    def sdk_time_in_force(self) -> Any:
        return map_enum(TimeInForceType, self.time_in_force)

    def sdk_outside_rth(self) -> Any:
        if self.outside_rth is None:
            return None
        return map_enum(OutsideRTH, self.outside_rth)


class CancelOrderInput(Model):
    order_id: str


class ReplaceOrderInput(Model):
    order_id: str
    quantity: Decimal
    price: Decimal | None = None
    trigger_price: Decimal | None = None
    limit_offset: Decimal | None = None
    trailing_amount: Decimal | None = None
    trailing_percent: Decimal | None = None
    limit_depth_level: int | None = None
    trigger_count: int | None = None
    monitor_price: Decimal | None = None
    remark: str | None = None


class CreateWatchlistGroupInput(Model):
    name: str
    securities: list[str] | None = None

    @field_validator("securities")
    @classmethod
    def _validate_securities(cls, value: list[str] | None) -> list[str] | None:
        if value is None:
            return value
        return [validate_symbol(item) for item in value]


class UpdateWatchlistGroupInput(CreateWatchlistGroupInput):
    id: int
    name: str | None = None
    mode: str | None = None

    def sdk_mode(self) -> Any:
        if self.mode is None:
            return None
        return map_enum(SecuritiesUpdateMode, self.mode)


class DeleteWatchlistGroupInput(Model):
    id: int
    purge: bool = False
